package org.deken.game.component;

import org.deken.game.input.InputMouseAction;
import org.deken.game.sprites.SpriteSize;

/**
 * <p>Title: GameContainer</p>
 *
 * <p>Description: The object is for storing other GameComponents, that the user
 * interacts with.</p>
 *
 * <p>Copyright: Copyright (c) 2006</p>
 *
 * <p>@author Gary Deken</p>
 *
 * @version 1.0
 */
public abstract class GContainer extends GComponent {
   protected boolean grabbed = false;

   /**
    * GameContainer
    * @param screenWidth
    * @param screenHeight
    */
   public GContainer(int screenWidth, int screenHeight) {
      this.screenWidth = screenWidth;
      this.screenHeight = screenHeight;
      size = new SpriteSize(screenWidth, screenHeight);
   }

   /**
    * @param screenWidth
    * @param screenHeight
    * @param inputMouseAction
    */
   public GContainer(int screenWidth, int screenHeight, InputMouseAction inputMouseAction) {
      super(inputMouseAction);
      this.screenWidth = screenWidth;
      this.screenHeight = screenHeight;
      size = new SpriteSize(screenWidth, screenHeight);
   }

   /**
    * @param screenWidth
    * @param screenHeight
    * @param inputMouseAction
    * @param width Width of the container
    * @param height Height of the container
    */
   public GContainer(int screenWidth, int screenHeight, InputMouseAction inputMouseAction, int width, int height) {
      super(inputMouseAction);
      this.screenWidth = screenWidth;
      this.screenHeight = screenHeight;
      size = new SpriteSize(width, height);
   }

   /**
    *
    * @param name
    * @return GameComponent
    */
   public abstract GComponent getComponentByName(String name);

   /**
    * Returns whether this container has been grabbed by a mouse or other input.
    * @return boolean
    */
   public boolean isGrabbed() {
      return grabbed;
   }

   /**
    *
    * @param gameComponent
    */
   public abstract void remove(GComponent gameComponent);

   /**
    * Sets whether this contain is grabbed by a mouse or other input.
    * @param grabbed
    */
   public void setGrabbed(boolean grabbed) {
      this.grabbed = grabbed;
   }

}
