/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.deken.game.component;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import org.deken.game.animation.Animation;
import org.deken.game.input.InputMouseAction;
import org.deken.game.map.GameLocation;
import org.deken.game.sprites.Sprite;
import org.deken.game.sprites.SpriteSize;

/**
 * <p>Title: GameLabel</p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) Mar 6, 2011</p>
 *
 * @author Gary Deken<
 * @version 1.0
 */
public class GLabel extends GTextContainer {
   private Animation animation;

   /**
    *
    * @param text
    */
   public GLabel(String text) {
      this(text, new Font(Font.SANS_SERIF, Font.PLAIN, 10), Color.black);
   }

   /**
    *
    * @param text
    * @param font
    * @param fontColor
    */
   public GLabel(String text, Font font, Color fontColor) {
      this(new GText(text, font, fontColor));
   }

   /**
    * @param gameText
    */
   public GLabel(GText gameText) {
      super();
      this.gameText = gameText;
      invalid = true;
   }

   /**
    *
    * @param animation
    */
   public GLabel(Animation animation) {
      super();
      this.animation = animation;
      if (animation != null) {
         size = new SpriteSize(animation.getWidth(), animation.getHeight());
      }
   }

   /**
    *
    * @return GameLabel
    */
   @Override
   public GLabel copy() {
      GLabel that;
      if (gameText == null) {
         that = new GLabel(this.animation.copy());
      }
      else {
         that = new GLabel(gameText);
      }
      that.size = this.size.copy();
      that.location = new GameLocation(this.location.x, this.location.y);
      that.gameText = this.gameText.copy();
      that.invalid = true;

      return that;
   }

   @Override
   public void draw(Graphics2D graphics, int xOffset, int yOffset) {
      if (invalid) {
         validate(graphics);
      }
      if (animation != null) {
         if (parent != null) {
            animation.draw(graphics, xOffset + (int) location.x, yOffset + (int) location.y);
         }
         else {
            animation.draw(graphics, (int) location.x, (int) location.y);
         }
      }
      if (gameText != null) {
         graphics.setColor(gameText.getFontColor());
         graphics.setFont(gameText.getFont());
         if (parent != null) {
            graphics.drawString(gameText.getText(), xOffset + (int) location.x, yOffset + (int) location.y + gameText.getTextOffset());
         }
         else {
            graphics.drawString(gameText.getText(), (int) location.x, (int) location.y + gameText.getTextOffset());
         }
      }
   }

   public GText getGameText() {
      return gameText;
   }

   public void notifyListener(InputMouseAction inputMouseAction) {
      throw new UnsupportedOperationException("Not supported yet.");
   }

   public void setGameText(GText gameText) {
      this.gameText = gameText;
      invalid = true;
   }

   /**
    * Convenience method for setting the text.
    *
    * @param text
    */
   public void setText(String text) {
      if (gameText != null) {
         gameText.setText(text);
      }
      invalid = true;
   }

   /**
    * @param elapseTime
    */
   @Override
   public void update(long elapseTime) {
      if (gameText == null) {
         animation.update(elapseTime);
      }
   }

   /**
    * @param graphics
    */
   public void validate(Graphics2D graphics) {
      if (gameText != null) {
         gameText.validate(graphics);
         // get size for string and font.
         size = gameText.getSize();
      }
      invalid = false;
      positionHorizontalAlignment();
      positionVerticalAlignment();
   }

   @Override
   protected void spriteCollison(Sprite collideSprite) {
      // Not used.
   }
}
