/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.deken.game.component;

import java.awt.Graphics2D;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import org.deken.game.input.InputAction;
import org.deken.game.input.InputListener;
import org.deken.game.input.InputMonitor;
import org.deken.game.input.InputMouseAction;
import org.deken.game.sprites.Sprite;

/**
 * <p>Title: GameMenu</p>
 *
 * <p>Description: This is a container object for storing a menu list, and
 * handling key movements up and down the menu.</p>
 *
 * <p>Copyright: Copyright (c) Mar 6, 2011</p>
 *
 * @author Gary Deken<
 * @version 1.0
 */
public class GMenu extends GContainer implements InputListener {
   public static final String DOWN_ARROW = "Down Arrow";
   public static final String UP_ARROW = "Up Arrow";
   private List<GTextContainer> menuList = new ArrayList<GTextContainer>();
   //private GameComponent.Alignment alignmentVertical = Alignment.CENTER;
   private int spacing = 0;
   private int menuInset = 0;
   private int index = 0;

   /**
    * Constructor
    * @param screenWidth
    * @param screenHeight
    */
   public GMenu(int screenWidth, int screenHeight) {
      super(screenWidth, screenHeight);
      setHorizontalAlignment(Alignment.CENTER, screenWidth);
      //setAlignmentVertical(Alignment.CENTER);
      InputAction keyUpAction = InputMonitor.getInputActionForKey(KeyEvent.VK_UP, UP_ARROW);
      keyUpAction.add(this);
      InputAction keyDownAction = InputMonitor.getInputActionForKey(KeyEvent.VK_DOWN, DOWN_ARROW);
      keyDownAction.add(this);
   }

   /**
    * @param screenWidth
    * @param screenHeight
    * @param alignmentHorizontal
    */
   public GMenu(int screenWidth, int screenHeight, Alignment alignmentHorizontal) {//, Alignment alignmentVertical) {
      this(screenWidth, screenHeight);
      setHorizontalAlignment(alignmentHorizontal, screenWidth);
      //setAlignmentVertical(alignmentVertical);
   }

   @Override
   public void update(long elapseTime) {
      for (int x = 0; x < menuList.size(); x++) {
         GTextContainer gameTextContainer = menuList.get(x);
         gameTextContainer.update(elapseTime);
      }
   }

   @Override
   public GMenu copy() {
      throw new UnsupportedOperationException("Not supported yet.");
   }

   /**
    * Calls the draw for all menu times, each time adding the button heights
    * spacing to the next button's Y location.
    * @param graphics
    * @param xOffset
    * @param yOffset
    */
   @Override
   public void draw(Graphics2D graphics, int xOffset, int yOffset) {
      if (invalid) {
         validate(graphics);
      }
      for (GTextContainer gameTextContainer : menuList) {
         gameTextContainer.draw(graphics, (int)location.x, (int)location.y);
      }
   }

   public void validate(Graphics2D graphics) {
      Iterator<GTextContainer> iter = menuList.iterator();
      while (iter.hasNext()) {
         iter.next().validate(graphics);
      }
      setLocations();
      invalid = false;
   }

   /**
    * Return the width of the menu, subtracting insets.
    * @return int
    */
   @Override
   public int getWidth() {
      int width = screenWidth;
      if (parent != null && parent.getWidth() > 0) {
         width = parent.getWidth();
      }
      width -= (menuInset * 2);
      return width;
   }

   /**
    * Return the combined GameButton heights.
    * @return int
    */
   @Override
   public int getHeight() {
      int height = 0;
      for (GTextContainer gameTextContainer : menuList) {
         height += gameTextContainer.getHeight() + spacing;
      }
      return height;
   }

   /**
    * Listen for the up and down arrow keys, to move the selection.
    * @param inputAction
    */
   public void notifyListener(InputAction inputAction) {
      if (UP_ARROW.equals(inputAction.getName())) {
         index++;
         if (index >= menuList.size()) {
            index = 0;
         }
      }
      else if (DOWN_ARROW.equals(inputAction.getName())) {
         index--;
         if (index < 0) {
            index = menuList.size() - 1;
         }
      }
//      GameTextContainer container = menuList.get(index);
//      menuList.get(index).setSelected();
   }

   public void notifyListener(InputMouseAction inputMouseAction) {
      throw new UnsupportedOperationException("Not supported yet.");
   }

   /**
    * Add a GameTextContainer at the given index.
    * @param gameTextContainer
    * @param index
    */
   public void add(GTextContainer gameTextContainer, int index) {
      menuList.add(index, gameTextContainer);
      gameTextContainer.setParent(this);
      invalid = true;
   }

   /**
    * Add a GameTextContainer at the end of the list.
    * @param gameTextContainer
    */
   public void add(GTextContainer gameTextContainer) {
      menuList.add(gameTextContainer);
      gameTextContainer.setParent(this);
      invalid = true;
   }

//   /**
//    * Sets the alignmentVertical for this GameMenu. That is the top to bottom
//    * alignment.
//    * @param alignment
//    */
//   public void setAlignmentVertical(Alignment alignment) {
//      this.alignmentVertical = alignment;
//      invalid = true;
//   }

   @Override
   public void remove(GComponent gameComponent) {
      if (gameComponent instanceof GTextContainer) {
         menuList.remove((GTextContainer)gameComponent);
      }
   }

   /**
    * Set the spacing between menu elements.
    * @param spacing
    */
   public void setSpacing(int spacing) {
      this.spacing = spacing;
   }

   public void setMenuInset(int menuInset) {
      this.menuInset = menuInset;
   }

   public GTextContainer getComponentByName(String name) {
      for (Iterator<GTextContainer> it = menuList.iterator(); it.hasNext();) {
         GTextContainer gameTextContainer = it.next();
         if (name.equals(gameTextContainer.getName())) {
            return gameTextContainer;
         }
      }
      return null;
   }

   @Override
   protected void spriteCollison(Sprite collideSprite) {
      // Not used.
   }

   private int getMenuWidth() {
      int width = 0;
      for (GTextContainer gameTextContainer : menuList) {
         if (gameTextContainer.getWidth() > width) {
            width = gameTextContainer.getWidth();
         }
      }
      return width;
   }

   private void setLocations() {
      int menuWidth = getMenuWidth();
      int width = getWidth();
      if (parent != null) {
         location.x = parent.getLocation().x + menuInset;
      }

      if (horizontalAlignment != null) {
         switch (horizontalAlignment) {
            case CENTER:
               location.x += (width / 2) - (menuWidth / 2);
               break;
            case RIGHT:
               location.x += width - menuInset - menuWidth;
               break;
            default:
               location.x = menuInset;
         }
      }
      int height = screenHeight;
      if (parent != null) {
         height = parent.getHeight();
      }
      if (verticalAlignment != null) {
         switch (verticalAlignment) {
            case CENTER:
               location.y = (height / 2) - (getHeight() / 2);
               break;
            case BOTTOM:
               location.y = height - menuInset - getHeight();
               break;
            default:
               location.y = menuInset;
         }
      }
      // set the location in each button.
      int yAdjLocation = 0;
      for (GTextContainer gameComponent : menuList) {
         switch(horizontalAlignment) {
            case LEFT:
               gameComponent.getLocation().setX(0);
               break;
            case CENTER:
               gameComponent.getLocation().setX((menuWidth / 2) - (gameComponent.getWidth() / 2));
               break;
            case RIGHT:
               gameComponent.getLocation().setX(menuWidth - menuInset - gameComponent.getWidth());
         }
         gameComponent.getLocation().setY(yAdjLocation);
         yAdjLocation += gameComponent.getHeight() + spacing;
      }
   }


}
