/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.deken.game.component;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;
import org.deken.game.input.InputMouseAction;
import org.deken.game.map.GameLocation;
import org.deken.game.sprites.Sprite;
import org.deken.game.sprites.SpriteSize;

/**
 * <p>Title: GameTextArea</p>
 *
 * <p>Description: </p>
 *
 * Copyright: Copyright (c) Feb 17, 2013
 *
 * @author Gary Deken
 * @version 1.0
 */
public class GParagraph extends GComponent {
   private static final String NEW_LINE = "\r";
   private String text;
   private Font font;
   private Color fontColor = Color.BLACK;
   private int lineHeight;
   private int lineSpacing = 0;
   private List<GText> textLines = new ArrayList<GText>();
   private boolean indentNewLines = false;
   private String indent = "";

   public GParagraph(String text, int width) {
      this(text, new Font(Font.SANS_SERIF, Font.PLAIN, 10), Color.black, width);
   }

   public GParagraph(String text, Font font, Color fontColor, int width) {
      super();
      this.text = text;
      this.font = font;
      this.fontColor = fontColor;
      invalid = true;
      size = new SpriteSize(width, 1);
   }

   /**
    * Adds a Blank line, with no text.
    */
   public void addBlankLine() {
      text += " " + NEW_LINE;
      invalid = true;
   }

   /**
    * Adds text as a new section. A carriage return is added before the new
    * text.
    *
    * @param textOnNewLine
    */
   public void addTextToSection(String textOnNewLine) {
      this.text += " " + NEW_LINE + textOnNewLine;
      invalid = true;
   }

   @Override
   public GComponent copy() {
      GParagraph that = new GParagraph(text, font, fontColor, size.getWidth());

      that.size = this.size.copy();
      that.location = new GameLocation(this.location.x, this.location.y);
      that.invalid = true;

      return that;
   }

   @Override
   public void draw(Graphics2D graphics, int xOffset, int yOffset) {
      if (invalid) {
         validate(graphics);
      }
      graphics.setColor(fontColor);
      graphics.setFont(font);
      int xLocation, yLocation;
      if (parent != null) {
         xLocation = xOffset + (int) location.x;
         yLocation = yOffset + (int) location.y;
      }
      else {
         xLocation = (int) location.x;
         yLocation = (int) location.y;
      }
      Iterator<GText> iter = textLines.iterator();
      while (iter.hasNext()) {
         GText gameText = iter.next();
         graphics.drawString(gameText.getText(), xLocation, yLocation + gameText.getTextOffset());
         yLocation += lineHeight + lineSpacing;
      }
   }

   public void notifyListener(InputMouseAction inputMouseAction) {
      throw new UnsupportedOperationException("Not supported yet.");
   }

   /**
    * Sets if new sections should be indented, and by how many spaces. This only
    * needs to be set once per GameParagraph
    *
    * @param indent
    * @param spaces
    */
   public void setIndentNewSection(boolean indent, int spaces) {
      this.indentNewLines = indent;
      for (int i = 0; i < spaces; i++) {
         this.indent += " ";
      }
   }

   /**
    * Sets the number of pixels to put between lines of text. The default is 0.
    *
    * @param lineSpacing
    */
   public void setLineSpacing(int lineSpacing) {
      this.lineSpacing = lineSpacing;
   }

   /**
    * Set the text value to be display in the paragraph. This over-rides any
    * current text.
    *
    * @param text
    */
   public void setText(String text) {
      this.text = text;
      invalid = true;
   }

   @Override
   public void update(long elapseTime) {
      // do nothing.
   }

   @Override
   public void validate(Graphics2D graphics) {
      textLines.clear();
      String str = buildTextLines(graphics);
      if (str.length() > 0) {
         GText gt = new GText(str, font, fontColor);
         gt.validate(graphics);
         textLines.add(gt);
      }
      invalid = false;
      size.setSize(size.getWidth(), lineHeight * textLines.size(), 1);
   }

   @Override
   protected void spriteCollison(Sprite collideSprite) {
      // not used
   }

   private String buildTextLines(Graphics2D graphics) {
      FontMetrics fontMetrics = graphics.getFontMetrics(this.font);
      lineHeight = fontMetrics.getMaxAscent();

      int width = 0;
      StringBuilder strBld = new StringBuilder();
      StringTokenizer tokenizer = new StringTokenizer(text, " ");
      while (tokenizer.hasMoreTokens()) {
         String str = tokenizer.nextToken() + " ";
         boolean newLine = str.startsWith(NEW_LINE);
         if (newLine) {
            str = str.substring(1);
            if (indentNewLines) {
               str = indent + str;
            }
         }
         int textWidth = fontMetrics.stringWidth(str);

         if (width + textWidth > size.getWidth() || newLine) {
            GText gt = new GText(strBld.toString(), font, fontColor);
            gt.validate(graphics);
            textLines.add(gt);
            strBld = new StringBuilder();
            width = 0;
         }
         width += textWidth;
         strBld.append(str);
      }
      return strBld.toString();
   }
}
