/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.deken.game.component;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.List;
import org.deken.game.input.InputMouseAction;
import org.deken.game.sprites.Sprite;
import org.deken.game.sprites.SpriteSize;

/**
 * <p>Title: GScrollingLabel</p>
 *
 * <p>Description: </p>
 *
 * Copyright: Copyright (c) Mar 24, 2013
 *
 * @author Gary Deken
 * @version 1.0
 */
public class GScrollingLabel extends GComponent {
   private String text;
   private Font font;
   private Color fontColor = Color.WHITE;
   private Color backgroundColor = Color.BLACK;
   private List<TextSize> textList = new ArrayList<TextSize>();
   private double speed;
   protected int startIndex = 0;
   protected double startSize = 0.0;
   protected double xTextOffset = 0;
   private boolean firstStart = false;
   private int widestWidth = 0;
   private int maxHeight = 0;

   /**
    *
    * @param text
    */
   public GScrollingLabel(String text, int width, double speed) {
      super();
      this.text = text;
      size = new SpriteSize(width, 1);
      this.speed = speed;
      this.font = new Font(Font.SANS_SERIF, Font.PLAIN, 10);
      invalid = true;
   }

   /**
    *
    * @param text
    * @param font
    * @param fontColor
    */
   public GScrollingLabel(String text, int width, double speed, Font font, Color fontColor) {
      super();
      this.text = text;
      size = new SpriteSize(width, 1);
      this.speed = speed;
      this.font = font;
      this.fontColor = fontColor;
      invalid = true;
   }

   public GScrollingLabel copy() {
      GScrollingLabel that = new GScrollingLabel(text, size.getWidth(), speed, font, fontColor);
      that.backgroundColor = this.backgroundColor;
      that.invalid = true;
      return that;
   }

   /**
    *
    * @param graphics
    * @param xOffset
    * @param yOffset
    */
   public void draw(Graphics2D graphics, int xOffset, int yOffset) {
      if (invalid) {
         validate(graphics);
      }
      int xLocation, yLocation;
      if (parent != null) {
         xLocation = xOffset + (int) location.x;
         yLocation = yOffset + (int) location.y;
      }
      else {
         xLocation = (int) location.x;
         yLocation = (int) location.y;
      }
      if (backgroundColor != null) {
         graphics.setColor(backgroundColor);
         graphics.fillRect(xLocation, yLocation - maxHeight, size.getWidth(), size.getHeight());
      }
      graphics.setColor(fontColor);
      graphics.setFont(font);
      int index = startIndex;
      int currentWidth = widestWidth - (int) startSize;
      StringBuilder str = new StringBuilder();

      while (currentWidth < size.getWidth()) {
         if (index >= textList.size()) {
            index = 0;
         }
         currentWidth += textList.get(index).getSize();
         if (currentWidth < size.getWidth()) {
            str.append(textList.get(index).getText());
            index++;
         }
         else {
            break;
         }
      }
      graphics.drawString(str.toString(), xLocation + widestWidth - (int) startSize, yLocation);

   }

   public Font getFont() {
      return font;
   }

   public Color getFontColor() {
      return fontColor;
   }

   public Color getBackgroundColor() {
      return backgroundColor;
   }

   public String getText() {
      return text;
   }

   public boolean isInvalid() {
      return invalid;
   }

   public void notifyListener(InputMouseAction inputMouseAction) {
      throw new UnsupportedOperationException("Not supported yet.");
   }

   public void setBackgroundColor(Color backgroundColor) {
      this.backgroundColor = backgroundColor;
   }

   public void setFont(Font font) {
      this.font = font;
      invalid = true;
   }

   public void setFontColor(Color fontColor) {
      this.fontColor = fontColor;
   }

   public void setText(String text) {
      this.text = text;
      invalid = true;
   }

   public void setInvalid(boolean invalid) {
      this.invalid = invalid;
   }

   public void setWidth(int width) {
      size = new SpriteSize(width, 1);
      invalid = true;
   }

   @Override
   public String toString() {
      return "GScrollingLabel: [" + text + "]; Font: " + font + "; Color" + fontColor;
   }

   /**
    *
    * @param elapseTime
    */
   public void update(long elapseTime) {
      if (firstStart) {
         // update the amount of movement during this sequence of time.
         xTextOffset = speed * elapseTime;
         startSize += xTextOffset;
         if (startSize > textList.get(startIndex).getSize()) {
            startSize -= textList.get(startIndex).getSize();
            startIndex++;
            if (startIndex >= textList.size()) {
               startIndex = 0;
            }
         }
      }
      else {
         firstStart = true;
         xTextOffset = 0;
      }
   }

   /**
    * Uses the current graphics object to get FontMetrics and the size of the
    * text.
    *
    * @param graphics
    */
   public void validate(Graphics2D graphics) {
      text += "   ";
      textList.clear();
      FontMetrics fontMetrics = graphics.getFontMetrics(this.font);
      widestWidth = (int) (fontMetrics.charWidth('W') * 0.75);
      char[] chars = text.toCharArray();
      for (int i = 0; i < chars.length; i++) {
         int width = fontMetrics.charWidth(chars[i]);
         textList.add(new TextSize(String.valueOf(chars[i]), width));
      }
      maxHeight = fontMetrics.getMaxAscent();
      int lineHeight = maxHeight + fontMetrics.getMaxDescent();
      size.setSize(size.getWidth(), lineHeight, 1);
      location.y = location.y + lineHeight;
      invalid = false;
   }

   protected void spriteCollison(Sprite collideSprite) {
      // Not used.
   }

   private class TextSize {
      private String text;
      private int size;

      public TextSize(String text, int size) {
         this.text = text;
         this.size = size;
      }

      public int getSize() {
         return size;
      }

      public String getText() {
         return text;
      }

      @Override
      public String toString() {
         return "Text: <" + text + "> Size: " + size;
      }
   }

}
