/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.deken.game.component;

import java.awt.Graphics2D;
import org.deken.game.input.InputMouseAction;
import org.deken.game.sprites.Sprite;
import org.deken.game.sprites.SpriteSize;

/**
 * <p>Title: GameGridContainer</p>
 *
 * <p>Description: </p>
 *
 * Copyright: Copyright (c) Feb 9, 2013
 *
 * @author Gary Deken
 * @version 1.0
 */
public class GTableContainer extends GContainer {
   private TableEntry[][] contents;
   private int rows;
   private int columns;
   private SpriteSize cellSize;

   /**
    *
    * @param screenWidth
    * @param screenHeight
    * @param columns
    * @param rows
    */
   public GTableContainer(int screenWidth, int screenHeight, int columns, int rows) {
      super(screenWidth, screenHeight);
      contents = new TableEntry[rows][columns];
      this.rows = rows;
      this.columns = columns;
      invalid = true;
      setCellSize();
   }

//   public GameTableContainer(int parentWidth, int parentHeight, int columns, int rows, SpriteSize size) {
//      super(screenWidth, screenHeight);
//      contents = new TableEntry[rows][columns];
//      invalid = true;
//
//   }
   @Override
   public void update(long elapseTime) {
      for (int row = 0; row < rows; row++) {
         for (int column = 0; column < columns; column++) {
            if (contents[row][column] != null) {
               contents[row][column].gameComponent.update(elapseTime);
            }
         }
      }
   }

   /**
    * Add a GameComponent to this GameTable, using the alignment for Horizontal
    * placement, in cell at the row and column. The alignment is used if it is
    * RIGHT, LEFT or CENTER. Vertical alignment is defaulted to CENTER. If there
    * is an entry already at the specified row and column, it is replaced with
    * this entry.
    *
    * @param gameComponent
    * @param horizontalAlignment
    * @param column
    * @param row
    */
   public void add(GComponent gameComponent, Alignment horizontalAlignment, int column, int row) {
      if (row > rows || column > columns || row < 0 || column < 0) {
         throw new IllegalArgumentException("Index out of range:  column: " + column + " row: " + row);
      }
      gameComponent.setParent(this);
      contents[row][column] = new TableEntry(gameComponent, horizontalAlignment, null);
      invalid = true;
   }

   /**
    * Add a GameComponent to this GameTable, using the alignment for Horizontal
    * and Vertical placement, in cell at the row and column. The alignment is
    * used if it is RIGHT, LEFT or CENTER. If a Vertical alignment is enter, the
    * horizontal defaults to CENTER. If there is an entry already at the
    * specified row and column, it is replaced with this entry.
    *
    * @param gameComponent
    * @param horizontalAlignment
    * @param verticalAlignment
    * @param column
    * @param row
    */
   public void add(GComponent gameComponent, Alignment horizontalAlignment, Alignment verticalAlignment, int column, int row) {
      if (row > rows || column > columns || row < 0 || column < 0) {
         throw new IllegalArgumentException("Index out of range:  column: " + column + " row: " + row);
      }
      gameComponent.setParent(this);
      contents[row][column] = new TableEntry(gameComponent, horizontalAlignment, verticalAlignment);
      invalid = true;
   }

   /**
    * Add a GameComponent to this GameTable, using the alignment for Horizontal
    * placement, in cell at the row and column. The alignment is defaulted to
    * Center alignment. If there is an entry already at the specified row and
    * column, it is replaced with this entry.
    *
    * @param gameComponent
    * @param column
    * @param row
    */
   public void add(GComponent gameComponent, int column, int row) {
      if (row > rows || column > columns || row < 0 || column < 0) {
         throw new IllegalArgumentException("Index out of range: column: " + column + "  row: " + row);
      }
      gameComponent.setParent(this);
      contents[row][column] = new TableEntry(gameComponent);
      invalid = true;
   }

   @Override
   public GComponent copy() {
      throw new UnsupportedOperationException("Not supported yet.");
   }

   @Override
   public void draw(Graphics2D graphics, int xOffset, int yOffset) {
      if (invalid) {
         validate(graphics);
         setLocations(xOffset, yOffset);
      }
      for (int row = 0; row < rows; row++) {
         for (int column = 0; column < columns; column++) {
            if (contents[row][column] != null) {
               contents[row][column].gameComponent.draw(graphics, (int) getLocation().x, (int) getLocation().y);
            }
         }
      }
   }

   public GComponent getComponentByName(String name) {
      for (int row = 0; row < rows; row++) {
         for (int column = 0; column < columns; column++) {
            if (contents[row][column] != null) {
               GComponent component = contents[row][column].gameComponent;
               if (name.equals(component.getName())) {
                  return component;
               }
            }
         }
      }
      return null;
   }

   public void notifyListener(InputMouseAction inputMouseAction) {
      throw new UnsupportedOperationException("Not supported yet.");
   }

   /**
    *
    * @param gameComponent
    */
   public void remove(GComponent gameComponent) {
      for (int row = 0; row < rows; row++) {
         for (int column = 0; column < columns; column++) {
            if (contents[row][column] != null) {
               GComponent component = contents[row][column].gameComponent;
               if (gameComponent.equals(component)) {
                  contents[row][column] = null;
                  break;
               }
            }
         }
      }
      invalid = true;
   }

   /**
    * Set the size of this container. If the height is zero, the cells will be
    * pushed up against each other.
    * @param spriteSize
    */
   @Override
   public void setSize(SpriteSize spriteSize) {
      super.setSize(spriteSize);
      setCellSize();
   }

   public void validate(Graphics2D graphics) {
      for (int row = 0; row < rows; row++) {
         for (int column = 0; column < columns; column++) {
            if (contents[row][column] != null) {
               contents[row][column].gameComponent.validate(graphics);
            }
         }
      }
      invalid = false;
   }

   @Override
   protected void spriteCollison(Sprite collideSprite) {
      // not used
   }

   protected void setLocations(int x, int y) {
      // set the X location of each element inside.
      int previousCellHeight = 0;
      for (int row = 0; row < rows; row++) {
         for (int column = 0; column < columns; column++) {
            TableEntry entry = contents[row][column];
            if (entry != null) {
               int newX, newY;
               GComponent component = contents[row][column].gameComponent;
               switch (entry.horizontalAlignment) {
                  case RIGHT: {
                     newX = x + (cellSize.getWidth() - component.getWidth());
                     break;
                  }
                  case CENTER: {
                     newX = x + (cellSize.getWidth() / 2) - (component.getWidth() / 2);
                     break;
                  }
                  default: {
                     // default to left.
                     newX = x;
                  }
               }
               int cellHeight;
               if (cellSize.getHeight() < 1) {
                  cellHeight = component.getHeight();
                  previousCellHeight = cellHeight;
               }
               else {
                  cellHeight = cellSize.getHeight();
               }
               switch (entry.verticalAlignment) {
                  case BOTTOM: {
                     newY = y + (cellHeight - component.getHeight());
                     break;
                  }
                  case CENTER: {
                     newY = y + (cellHeight / 2) - (component.getHeight() / 2);
                     break;
                  }
                  default: {
                     newY = y;
                  }
               }
               component.getLocation().setX(newX);
               component.getLocation().setY(newY);
            }
            x += cellSize.getWidth();
         }
         x = 0;
         if (cellSize.getHeight() < 1) {
            y += previousCellHeight;
         }
         else {
            y += cellSize.getHeight();
         }
      }
   }

   /**
    * Set the size of the individual cells. This container's size is used,
    * unless there is a parent. Then the parent size is used to determine this
    * container's size.
    */
   private void setCellSize() {
      int totalHeight = size.getHeight();
      int totalWidth = size.getWidth() < 1 ? 0 : size.getWidth();
      if (parent != null) {
         totalHeight = parent.getHeight();
         totalWidth = parent.getWidth();
      }
      cellSize = new SpriteSize(totalWidth / columns, totalHeight / rows);
   }

   private class TableEntry {
      protected GComponent gameComponent;
      protected Alignment horizontalAlignment = Alignment.CENTER;
      protected Alignment verticalAlignment = Alignment.CENTER;

      public TableEntry(GComponent gameComponent, Alignment horizontalAlignment, Alignment verticalAlignment) {
         this.gameComponent = gameComponent;
         if (horizontalAlignment != null) {
            this.horizontalAlignment = horizontalAlignment;
         }
         if (verticalAlignment != null) {
            this.verticalAlignment = verticalAlignment;
         }
      }

      public TableEntry(GComponent gameComponent) {
         this.gameComponent = gameComponent;
      }
   }

}
